local Abacus = AceLibrary("Abacus-2.0");

CharacterInfo = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
CharacterInfo.version = "1.0." .. string.sub("$Revision: 63453 $", 12, -3);
CharacterInfo.date = string.sub("$Date: 2008-03-03 06:16:55 -0500 (Mon, 03 Mar 2008) $", 8, 17);
CharacterInfo:RegisterDB("CharacterInfoDB");
CharacterInfo:RegisterDefaults('profile', {
	ViewBags = true,
	ViewBank = true,
	ViewMail = true,
});
	
CharacterInfo.vars = {}
CharacterInfo.vars.slotNames = {}

local DAYTIME = 24*60*60;
local EXPTIME = 30*DAYTIME;
	
function CharacterInfo:OnInitialize()
--	self:RegisterChatCommand({"/CharacterInfo", "/ci"}, {
--		desc = 'View inventory, bags and bank for all your characters',
--		handler = CharacterInfo,
--		type = 'group',
--		args = {
--			menu = {
--				name = 'Menu',
--				desc = 'Show CharacterInfo window',
--				type = 'execute',
--				func = 'Character_Toggle',
--			},
--			bags = {
--				name = 'ViewBags',
--				desc = 'View characters bag content',
--				type = 'toggle',
--				get = function() return self.db.profile.ViewBags end,
--				set = function(val) self.db.profile.ViewBags = val end,
--			},
--			bank = {
--				name = 'ViewBank',
--				desc = 'View characters bank content (including bank bags)',
--				type = 'toggle',
--				get = function() return self.db.profile.ViewBank end,
--				set = function(val) self.db.profile.ViewBank = val end,
--			},
--		},
--	});
	self.vars.playerName = UnitName("player");
end

function CharacterInfo:OnEnable()
	self.vars.selectedCharacter = self.vars.playerName;
	UIDropDownMenu_SetText(self.vars.selectedCharacter, CharacterInfoCharacterSelection);
	self:UpdateFramePosition();
	self:UpdateFrameStrata();
	self:UpdateFrameScale();
end

function CharacterInfo:OnDisable()
end

function CharacterInfo:Character_Show()
	self:UpdateFramePosition();
	self:UpdateFrameStrata();
	self:UpdateFrameScale();
	ShowUIPanel(CharacterInfoFrame);
end

function CharacterInfo:Character_Hide()
	HideUIPanel(CharacterInfoFrame);
end

function CharacterInfo:Character_Toggle()
	if CharacterInfoFrame:IsVisible() then
		self:Character_Hide()
	else
		self:Character_Show()
	end
end

function CharacterInfo:UpdateFramePosition()
	if CharacterInfoStorage.db.profile.SavePosition and CharacterInfoStorage.db.profile.PositionX and CharacterInfoStorage.db.profile.PositionY then
		local s = CharacterInfoFrame:GetEffectiveScale();
		CharacterInfoFrame:ClearAllPoints();
		CharacterInfoFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", 
					CharacterInfoStorage.db.profile.PositionX / s, CharacterInfoStorage.db.profile.PositionY / s);
	end
end

function CharacterInfo:SaveFramePosition()
	if CharacterInfoStorage.db.profile.SavePosition then
		local s = CharacterInfoFrame:GetEffectiveScale();
		CharacterInfoStorage.db.profile.PositionX = CharacterInfoFrame:GetLeft() * s;
		CharacterInfoStorage.db.profile.PositionY = CharacterInfoFrame:GetTop() * s;
	end
end

function CharacterInfo:UpdateFrameStrata()
	if CharacterInfoStorage.db.profile.FrameStrata then
		CharacterInfoFrame:SetFrameStrata(CharacterInfoStorage.db.profile.FrameStrata);
		CharacterInfoBagsFrame:SetFrameStrata(CharacterInfoStorage.db.profile.FrameStrata);
		CharacterInfoBankFrame:SetFrameStrata(CharacterInfoStorage.db.profile.FrameStrata);
		CharacterInfoMailFrame:SetFrameStrata(CharacterInfoStorage.db.profile.FrameStrata);
	end
end

function CharacterInfo:UpdateFrameScale()
	if CharacterInfoStorage.db.profile.Scale then
		CharacterInfoFrame:SetScale(CharacterInfoStorage.db.profile.Scale);
		CharacterInfoBagsFrame:SetScale(CharacterInfoStorage.db.profile.Scale);
		CharacterInfoBankFrame:SetScale(CharacterInfoStorage.db.profile.Scale);
		CharacterInfoMailFrame:SetScale(CharacterInfoStorage.db.profile.Scale);
		self:UpdateFramePosition()
	end
end

function CharacterInfo:Character_OnShow()
	self:Character_Update()
end

function CharacterInfo:Character_OnHide()
	CharacterInfoBagsFrame:Hide();
	CharacterInfoBankFrame:Hide();
	CharacterInfoMailFrame:Hide();
end

function CharacterInfo:Bags_OnClick()
	self.db.profile.ViewBags = not self.db.profile.ViewBags;
	self:Character_Update();
end

function CharacterInfo:Bank_OnClick()
	self.db.profile.ViewBank = not self.db.profile.ViewBank;
	self:Character_Update();
end

function CharacterInfo:Mail_OnClick()
	self.db.profile.ViewMail = not self.db.profile.ViewMail;
	self:Character_Update();
end

function CharacterInfo:ItemSlotButton_OnLoad()
--	local text = "ItemSlotButton_OnLoad"..this:GetName()..strsub(this:GetName(),14);
--	DEFAULT_CHAT_FRAME:AddMessage(text);
	local slotName = this:GetName();
	local id, textureName, checkRelic = GetInventorySlotInfo(strsub(slotName,14));
	self.vars.slotNames[id]=slotName;
	this:SetID(id);
	local texture = getglobal(slotName.."IconTexture");
	texture:SetTexture(textureName);
	this.backgroundTextureName = textureName;
	this.checkRelic = checkRelic;
end

function CharacterInfo:ItemSlotButton_OnEnter()
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	if this.hasItem then
		if this.known then
			GameTooltip:SetHyperlink(this.tooltip);	
		else
			GameTooltip:SetText(self.name);
			GameTooltip:AddLine("Item not in local cache");
			GameTooltip:AddLine("Click to try to update local cache.");
			GameTooltip:AddLine("Warning! You can be disconnected.");
			GameTooltip:Show();
		end
	else
		local text = TEXT(getglobal(strupper(strsub(this:GetName(), 14))));
		if ( this.checkRelic and UnitHasRelicSlot("player") ) then
			text = TEXT(getglobal("RELICSLOT"));
		end
		GameTooltip:SetText(text);
	end	
	CursorUpdate();
end

function CharacterInfo:ContainerItemButton_OnEnter()
	if this.item and this.hasItem then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		local mailSplit = {strsplit("\n",this.item)}
		if #mailSplit > 1 then
			GameTooltip:SetText(("Multiple Items (%d)"):format(#mailSplit))
			for _,val in pairs(mailSplit) do
				local _,_,id,i = string.find(val,"([^|]+)|(%d*)|?.*")
				local _,itemLink,_,_,_,_,_,_,_,itemTexture = GetItemInfo("item:"..id)					
				if itemLink then
					if i and tonumber(i) > 1 then
						GameTooltip:AddLine(("%sx%d"):format(itemLink, i))
					else
						GameTooltip:AddLine(itemLink)
					end
						GameTooltip:AddTexture(itemTexture)
				else
					GameTooltip:AddLine("Unknown Item")
				end
			end
		else
			GameTooltip:SetHyperlink(this.tooltip);	
		end
		GameTooltip:AddLine(this.subject);
		GameTooltip:AddLine("Sender: "..this.sender);
		if this.money then
			if this.money > 0 then
				GameTooltip:AddLine("Money: "..Abacus:FormatMoneyFull(this.money,true));
			elseif this.money < 0 then
				GameTooltip:AddLine("COD: "..Abacus:FormatMoneyFull(-this.money,true));
			end
		end
		if this.expTime then
			GameTooltip:AddLine(self:ExpiresTime(this.expTime,this.returnable));
		end
		GameTooltip:Show();
	elseif this.subject then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		GameTooltip:SetText(this.subject);
		GameTooltip:AddLine("Sender: "..this.sender);
		if this.money then
			if this.money > 0 then
				GameTooltip:AddLine("Money: "..Abacus:FormatMoneyFull(this.money,true));
			elseif this.money < 0 then
				GameTooltip:AddLine("COD: "..Abacus:FormatMoneyFull(-this.money,true));
			end
		end
		if this.expTime then
			GameTooltip:AddLine(self:ExpiresTime(this.expTime,this.returnable));
		end
		GameTooltip:Show();
	else
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		if this.known then
			GameTooltip:SetHyperlink(this.tooltip);	
		else
			GameTooltip:SetText(self.name);
			GameTooltip:AddLine("Item not in local cache");
			GameTooltip:AddLine("Click to try to update local cache.");
			GameTooltip:AddLine("Warning! You can be disconnected.");
			GameTooltip:Show();
		end
	end
	CursorUpdate();
end

function CharacterInfo:ExpiresTime(expTime, returnable)
	if expTime < 0 then
		expTime = -expTime;
	end
	local inTime = expTime-time();
	local msg
	if inTime < 0 then
		if returnable==1 then msg="Returned since: " else msg="Expired since: " end
		return RED_FONT_COLOR_CODE..msg..SecondsToTime(-inTime)..FONT_COLOR_CODE_CLOSE;
	end
	if inTime > EXPTIME then
		return "Delivered in: "..SecondsToTime(inTime-EXPTIME);
	end
	if returnable==1 then msg="Returned in: " else msg="Expires in: " end
	return msg..SecondsToTime(inTime);
end

function CharacterInfo:ItemButton_OnClick(button)
	if this.hasItem then
		if ( button == "LeftButton" ) then
			if ( IsControlKeyDown() ) then
				DressUpItemLink(this.tooltip);
			elseif IsShiftKeyDown() then 
				if ChatFrameEditBox:IsShown() then
					if this.link then
						ChatFrameEditBox:Insert(this.link);
					end
				end
			else --if not this.link then
				if this.item then
					for _,val in pairs({strsplit("\n",this.item)}) do
						local _,_,id = string.find(val,"([^|]+)|%d*|?.*")
						GameTooltip:SetOwner(this, "ANCHOR_TOPLEFT");
						GameTooltip:SetHyperlink("item:"..id);
					end
				else
					GameTooltip:SetOwner(this, "ANCHOR_TOPLEFT");
					GameTooltip:SetHyperlink(this.tooltip);
				end
				self:ScheduleEvent(self.Character_Update, 1, self);
			end
		end
	end
end
---------------------
--Init a Dropdown
function CharacterInfo:DropDown_Initialize()
	--self:Print("DropDown_Initialize",this:GetName());
	if (this:GetName() == "CharacterInfoCharacterSelectionButton") then
		local tnames = {};
		for name in pairs(CharacterInfoStorage.db.realm) do
			table.insert(tnames,name);
		end
		table.sort(tnames)		
		local info = { func = self.CharacterDropDown_OnClick };
		for _,name in ipairs(tnames) do
			info.text = name;
			info.checked = nil;
			UIDropDownMenu_AddButton(info);
		end
	end
end

---------------------
-- Dropdown Onclick
function CharacterInfo.CharacterDropDown_OnClick()
	local self = CharacterInfo;
	--self:Print("CharacterDropDown_OnClick",this:GetID());
	local ddl = getglobal("CharacterInfoCharacterSelection");
	UIDropDownMenu_SetSelectedID(ddl, this:GetID());
	self.vars.selectedCharacter = this:GetText();
	self:Character_Update();
end

-- Draw windows for a character
function CharacterInfo:Character_Update()
--	debugprofilestart();
	CharacterInfoNameText:SetText(self.vars.selectedCharacter);
	CharacterInfoLevelText:SetText(	string.format("Level %s %s %s", 
			CharacterInfoStorage.db.realm[self.vars.selectedCharacter].level,
			CharacterInfoStorage.db.realm[self.vars.selectedCharacter].race,
			CharacterInfoStorage.db.realm[self.vars.selectedCharacter].class ));
	for i=1,19 do
		local itemSlot = getglobal(self.vars.slotNames[i]);
		local name,itemString,link,texture,known,num,hasItem = self:GetItemInfo(CharacterInfoStorage.db.realm[self.vars.selectedCharacter].inventory[i]);
		if hasItem then
			SetItemButtonTexture(itemSlot,texture);
			itemSlot.hasItem = true;
			itemSlot.tooltip = itemString;
			itemSlot.link = link;
			itemSlot.known = known;
		else
			itemSlot.hasItem = false;
			itemSlot.link = nil;
			itemSlot.known = nil;
			SetItemButtonTexture(itemSlot,itemSlot.backgroundTextureName);
		end
	end	
	if self.db.profile.ViewBags then
		self:DrawContainer(0);
	else
		CharacterInfoBagsFrame:Hide();
	end
	if self.db.profile.ViewBank then
		self:DrawContainer(1);
	else
		CharacterInfoBankFrame:Hide();
	end
	if self.db.profile.ViewMail then
		self:DrawContainer(2);
	else
		CharacterInfoMailFrame:Hide();
	end
	t = debugprofilestop()
--	self:Print("Character_Update %f",t);
end

function CharacterInfo:GetItemInfo(inf)
	if not inf then return end
	local itemString,name,itemLink,texture
	local known = true;
	local hasItem;
	local id,num,sender,expTime,returnable,subject,money = CharacterInfoStorage:SplitItemInfo(inf);
	if id then
		itemString = "item:"..id;
		hasItem = true;
		name,itemLink,_,_,_,_,_,_,_,texture = GetItemInfo(itemString);
	else
		texture = "Interface\\Icons\\INV_Scroll_03";
		name = subject;
		known = false;
	end
	if not name then
		name = "Unknown Item";
		itemLink = nil;
		texture = "Interface\\Icons\\INV_Misc_QuestionMark.blp"
		known = false;
	end
	return name,itemString,itemLink,texture,known,num,hasItem,sender,expTime,returnable,subject,money;
end

function CharacterInfo:GetItemButton(frame,index)
	local buttonName = frame:GetName().."Item"..index;
	local itemButton = getglobal(buttonName);
	if not itemButton then
		itemButton = CreateFrame("Button", buttonName, frame, "CharacterInfoContainerButtonTemplate");
	end
	itemButton:ClearAllPoints();
	return itemButton
end

function CharacterInfo:SetItemButtomItem(itemButton, item, mail)
	if item then
		local name,itemString,link,texture,known,num,hasItem,sender,expTime,returnable,subject,money = self:GetItemInfo(strsplit("\n",item));
		SetItemButtonTexture(itemButton, texture);
		SetItemButtonCount(itemButton, num);
		itemButton.name = name;
		itemButton.hasItem = hasItem;
		itemButton.tooltip = itemString;
		itemButton.link = link;
		itemButton.known = known;
		itemButton.sender = sender;
		itemButton.expTime = expTime;
		itemButton.returnable = returnable;
		itemButton.subject = subject;
		itemButton.money = money;
		if mail then itemButton.item = item; end
		local c = {r=0,g=0,b=0}
		if expTime then
			local inTime = math.abs(expTime)-time();
			if inTime > EXPTIME then
				c = {r=0,g=0.82,b=1}
			elseif inTime < CharacterInfoStorage.db.profile.WarningDays*DAYTIME then
				if returnable == 1 then
					c = {r=1,g=0.82,b=0}
				else
					c = {r=1,g=0,b=0}
				end
			end
		end
		getglobal(itemButton:GetName().."Slot"):SetVertexColor(c.r, c.g, c.b);		
	else
		SetItemButtonTexture(itemButton, "");
		SetItemButtonCount(itemButton, nil);
		itemButton.hasItem = false;
		itemButton.tooltip = nil;
		itemButton.link = nil;
		itemButton.known = nil;
		itemButton.sender = nil;
		itemButton.expTime = nil;
		itemButton.returnable = nil;
		itemButton.subject = nil;
		itemButton.money = nil;
		getglobal(itemButton:GetName().."Slot"):SetVertexColor(0, 0, 0);		
	end
	itemButton:Show();
end

function CharacterInfo:DrawContainer(id)
	local frame;
	local frameName;
	local container;
	local size;
	local firstRow = 0;
	if ( id == 0 ) then  -- Bags
		frame = getglobal("CharacterInfoBagsFrame");
		frameName = frame:GetName();
		getglobal(frameName.."Name"):SetText("Bags");
		container = CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bags;
		size = 16;
		firstRow = 48;
		for j=1, 4 do
			local _,_,_,_,_,num = self:GetItemInfo(CharacterInfoStorage.db.realm[self.vars.selectedCharacter].inventory[ContainerIDToInventoryID(j)]);
			if num then size = size + num end
		end
	elseif ( id == 1 ) then  -- Bank
		frame = getglobal("CharacterInfoBankFrame");
		frameName = frame:GetName();
		getglobal(frameName.."Name"):SetText("Bank");
		container = CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bank;
		size = 28;
		firstRow = 48;
		for j=5, 11 do
			local _,_,_,_,_,num = self:GetItemInfo(CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bank[ContainerIDToInventoryID(j)]);
			if num then size = size + num end
		end
	elseif ( id == 2 ) then  -- Mail
		frame = getglobal("CharacterInfoMailFrame");
		frameName = frame:GetName();
		getglobal(frameName.."Name"):SetText("Mailbox");
		container = CharacterInfoStorage.db.realm[self.vars.selectedCharacter].mail;
		if container then
			size = #container;
		else
			size = 0;
		end
	else
		return;
	end
	if size == 0 then
		frame:Hide();
		return
	end
	local items = {};
	for key in pairs(container) do
		if key < 60 or key > 100 then
			table.insert(items,key);
		end
	end
	table.sort(items,function(a,b) 
		a = 50-mod(a,100)+100*floor(a/100)
		b = 50-mod(b,100)+100*floor(b/100)
		return tonumber(a)<tonumber(b) 
		end);
	local rows = ceil(math.sqrt(size));
	local columns = ceil(size / rows);
	if ( columns < 4 ) then 
		columns = 4;
		rows = ceil(size / columns);
	end
	if ( id == 1 and columns < 7 ) then 
		columns = 7;
		rows = ceil(size / columns);
	end
	if columns > 8 then
		columns = 8;
		rows = ceil(size / columns);
	end
  local spacing = 5
  local iconsize = 37
  local iconspace = spacing+iconsize
	frame:SetHeight(rows * iconspace + 26 + firstRow);
  frame:SetWidth(columns * iconspace + 12);
	for j=1,size do
		local itemButton = self:GetItemButton(frame,j);
		-- Set first button
		if( j == 1 ) then
			itemButton:SetPoint("BOTTOMRIGHT", frameName, "BOTTOMRIGHT", -8, 8+firstRow);
		else
			if ( mod((j-1), columns) == 0 ) then
				itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(j - columns), "TOPRIGHT", 0, spacing);
			else
				itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(j - 1), "BOTTOMLEFT", -spacing, 0);
			end
		end
		if ( items[j] ) then
			if id == 2 then
				self:SetItemButtomItem(itemButton, container[items[j]], true);
			else
				self:SetItemButtomItem(itemButton, container[items[j]]);
			end
		else
			self:SetItemButtomItem(itemButton);
		end
	end
	if ( id == 0 ) then
		for j=1,4 do
			local itemButton = self:GetItemButton(frame,j+size);
			if ( j == 1 ) then
				itemButton:SetPoint("BOTTOMRIGHT", frameName, "BOTTOMRIGHT", -8, 8);
			else
				itemButton:SetPoint("BOTTOMRIGHT", frameName.."Item"..(j + size - 1), "BOTTOMLEFT", -spacing, 0);
			end
			self:SetItemButtomItem(itemButton, CharacterInfoStorage.db.realm[self.vars.selectedCharacter].inventory[ContainerIDToInventoryID(j)]);
		end
	elseif ( id == 1 ) then
		for j=1,7 do
			local itemButton = self:GetItemButton(frame,j+size);
			if ( j == 1 ) then
				itemButton:SetPoint("BOTTOMLEFT", frameName, "BOTTOMLEFT", 8, 8);
			else
				itemButton:SetPoint("BOTTOMLEFT", frameName.."Item"..(j + size - 1), "BOTTOMRIGHT", spacing, 0);
			end
			self:SetItemButtomItem(itemButton, CharacterInfoStorage.db.realm[self.vars.selectedCharacter].bank[ContainerIDToInventoryID(4+j)]);
		end
	end
	j = size + 1 + (((id == 0) and 5) or ((id == 1) and 8) or 0);
	while true do
		local itemButton = getglobal(frameName.."Item"..j);
		if itemButton then
			itemButton:Hide();
		else
			break;
		end
		j = j + 1;
	end
	if ( id == 0 ) then
		frame:SetPoint("TOPLEFT", "CharacterInfoFrame", "TOPRIGHT", 4, 0 );
	elseif ( id == 1 ) then
		if CharacterInfoBagsFrame:IsVisible() then
			frame:SetPoint("TOPLEFT", "CharacterInfoBagsFrame", "TOPRIGHT", 4, 0 );
		else
			frame:SetPoint("TOPLEFT", "CharacterInfoFrame", "TOPRIGHT", 4, 0 );
		end
	elseif ( id == 2 ) then
		if CharacterInfoBankFrame:IsVisible() then
			frame:SetPoint("TOPLEFT", "CharacterInfoBankFrame", "TOPRIGHT", 4, 0 );
		elseif CharacterInfoBagsFrame:IsVisible() then
			frame:SetPoint("TOPLEFT", "CharacterInfoBagsFrame", "TOPRIGHT", 4, 0 );
		else
			frame:SetPoint("TOPLEFT", "CharacterInfoFrame", "TOPRIGHT", 4, 0 );
		end
	end
	frame:Show();
end
